/*
 *  sched-getattr.c - test program for the new sched_getattr() syscall
 *
 *  Copyright (C) 2014 Christophe Blaess <christophe.blaess@logilin.fr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sched.h>
#include <stdio.h>
#include <stdlib.h>
#include <schedattr.h>
#include <unistd.h>

int main(int argc, char * argv [])
{
	int i;
	int pid;
	struct sched_attr attr;

	for (i = 1; i < argc; i ++) {
		if (sscanf(argv[i], "%d", & pid) != 1) {
			fprintf(stderr, "usage: %s pid...\n", argv[0]);
			exit(EXIT_FAILURE);
		}
		if (sched_getattr(pid, & attr, sizeof(struct sched_attr), 0) != 0) {
			perror(argv[i]);
			continue;
		}
		printf("%d: ", pid);
		if (attr.sched_policy == SCHED_FIFO) {
			printf("Realtime FIFO scheduling, priority=%d\n", attr.sched_priority);
			continue;
		}
		if (attr.sched_policy == SCHED_RR) {
			printf("Realtime RR scheduling, priority=%d\n", attr.sched_priority);
			continue;
		}
		if (attr.sched_policy == SCHED_DEADLINE) {
			printf("Deadline scheduling, runtime=%lu, deadline=%lu, period=%lu\n",
			       attr.sched_runtime, attr.sched_deadline, attr.sched_period);
			continue;
		}
		printf("Timesharing scheduling, nice=%d\n", attr.sched_nice);
	}
	return EXIT_SUCCESS;
}

