/*
 *  sched-setattr.c - test program for the new sched_setattr() syscall
 *
 *  Copyright (C) 2014 Christophe Blaess <christophe.blaess@logilin.fr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sched.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <schedattr.h>
#include <unistd.h>

void usage_exit(const char * name)
{
	fprintf(stderr, "usage: %s R <priority> <pid>        (RR)\n", name);
	fprintf(stderr, "          F <priority> <pid>        (Fifo)\n");
	fprintf(stderr, "          D <Runtime> <Dealine> <Period> <pid>\n");
	fprintf(stderr, "          O <nice> <pid>            (Other)\n");
	exit(EXIT_FAILURE);
}


int main(int argc, char * argv [])
{
	int i;
	int pid;
	struct sched_attr attr;

	i = 0;
	while (++i < argc) {

		memset(& attr, 0, sizeof(attr));

		if (strcmp(argv[i], "R") == 0) {
			attr.sched_policy = SCHED_RR;
			if (++i == argc) {
				fprintf(stderr, "missing priority...\n");
				usage_exit(argv[0]);
			}
			if (sscanf(argv[i], "%d", & (attr.sched_priority)) != 1) {
				fprintf(stderr, "wrong priority...\n");
				usage_exit(argv[0]);
			}
		} else if (strcmp(argv[i], "F") == 0) {
			attr.sched_policy = SCHED_FIFO;
			if (++i == argc) {
				fprintf(stderr, "missing priority...\n");
				usage_exit(argv[0]);
			}
			if (sscanf(argv[i], "%d", & (attr.sched_priority)) != 1) {
				fprintf(stderr, "wrong priority...\n");
				usage_exit(argv[0]);
			}
		} else if (strcmp(argv[i], "O") == 0) {
			attr.sched_policy = SCHED_OTHER;
			if (++i == argc) {
				fprintf(stderr, "missing nice value...\n");
				usage_exit(argv[0]);
			}
			if (sscanf(argv[i], "%d", & (attr.sched_nice)) != 1) {
				fprintf(stderr, "wrong nice value...\n");
				usage_exit(argv[0]);
			}
		} else if (strcmp(argv[i], "D") == 0) {
			attr.sched_policy = SCHED_DEADLINE;
			if (++i == argc) {
				fprintf(stderr, "missing runtime value...\n");
				usage_exit(argv[0]);
			}
			if (sscanf(argv[i], "%lu", & (attr.sched_runtime)) != 1) {
				fprintf(stderr, "wrong runtime value...\n");
				usage_exit(argv[0]);
			}
			if (++i == argc) {
				fprintf(stderr, "missing deadline value...\n");
				usage_exit(argv[0]);
			}
			if (sscanf(argv[i], "%lu", & (attr.sched_deadline)) != 1) {
				fprintf(stderr, "wrong deadline value...\n");
				usage_exit(argv[0]);
			}
			if (++i == argc) {
				fprintf(stderr, "missing period value...\n");
				usage_exit(argv[0]);
			}
			if (sscanf(argv[i], "%lu", & (attr.sched_period)) != 1) {
				fprintf(stderr, "wrong period value...\n");
				usage_exit(argv[0]);
			}
		} else {
			fprintf(stderr, "unknown %s scheduling...\n", argv[i]);
			usage_exit(argv[0]);
		}

		if (++i == argc) {
			fprintf(stderr, "missing pid...\n");
			usage_exit(argv[0]);
		}
		if (sscanf(argv[i], "%d", & pid) != 1) {
			fprintf(stderr, "wrong pid...\n");
			usage_exit(argv[0]);
		}
		if (sched_setattr(pid, & attr, 0) != 0) {
			perror(argv[i]);
		}
	}
		
	return EXIT_SUCCESS;
}

