/*
 *  schedattr.c - new sched_getattr() & sched_setattr() syscall access
 *
 *  Copyright (C) 2014 Christophe Blaess <christophe.blaess@logilin.fr>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define _GNU_SOURCE
#include <sched.h>
#include <unistd.h>
#include <sys/syscall.h>

#ifndef SCHED_DEADLINE

#include <schedattr.h>


// Definitions pour x86_64
#define SCHED_SETATTR_SYSCALL_NB 314
#define SCHED_GETATTR_SYSCALL_NB 315

// Definitions pour i386
//#define SCHED_SETATTR_SYSCALL_NB 351
//#define SCHED_GETATTR_SYSCALL_NB 352


long sched_getattr(pid_t pid, struct sched_attr * attr, unsigned int size, unsigned int flags)
{
	return syscall(SCHED_GETATTR_SYSCALL_NB, pid, attr, size, flags);
}

#define SCHED_FLAG_RESET_ON_FORK  1

long sched_setattr(pid_t pid, struct sched_attr * attr, unsigned int flags)
{
	if (attr->sched_policy == SCHED_DEADLINE) {
		attr->sched_priority = 0;
		attr->sched_flags |= SCHED_FLAG_RESET_ON_FORK;
	}
	return syscall(SCHED_SETATTR_SYSCALL_NB, pid, attr, flags);
}

#endif

